#!/usr/bin/env python
#coding: utf-8
"""
BaseObject

Object를 dict처럼 사용할 수 있게 합니다.
또한, runtime에 object의 새로운 attribute를 생성할 수 있게 합니다.

>>> testobj = BaseObject()
>>> testobj.a
Traceback (most recent call last):
AttributeError: BaseObject instance has no attribute 'a'
>>> testobj.a = 1
>>> testobj.a
1
>>> testobj['a']
1
>>> testobj['b']
Traceback (most recent call last):
KeyError: 'b'
>>> testobj['b'] = 2
>>> testobj['b']
2
>>> testobj.b
2
"""

from warnings import warn

class BaseObject:
	def __str__(self):
		"""
		x.__str__() <==> str(x)
		"""
		return self.__repr__()

	def __setattribute__(self, key, value):
		"""
		x.__setattribute__(k, v) <==> x.k = v
		This cause a warning when k is introduced first time.
		"""
		if not hasattr(self, key):
			warn("'%s' is defined here first time." % key, RuntimeWarning, 2)
		self.__setitem__(key, value)

	def __getitem__(self, key):
		"""
		x.__getitem__(k) <==> x[k]
		"""
		import traceback
		if not hasattr(self, key):
			raise KeyError, key
		return getattr(self, key)

	def __setitem__(self, key, value):
		"""
		x.__setitem__(k, v) <==> x[k]=v
		This cause a warning when k is introduced first time.
		"""
		if not hasattr(self, key):
			warn("'%s' is defined here first time." % key, RuntimeWarning, 2)
		setattr(self, key, value)


if __name__ == "__main__":
        import doctest
        doctest.testmod()
